@@ -2,8 +2,8 @@
 **
 **     get_cpuinfo.h
 **
+** Copyright (c) 1993-2001 Marcel Mol, All Rights Reserved.
 ** Copyright (c) 1991-1995 Jussi Maki, All Rights Reserved.
-** Copyright (c) 1993-1998 Marcel Mol, All Rights Reserved.
 ** NON-COMMERCIAL USE ALLOWED. YOU ARE FREE TO DISTRIBUTE
 ** THIS PROGRAM AND MODIFY IT AS LONG AS YOU KEEP ORIGINAL
 ** COPYRIGHTS.
@@ -14,7 +14,7 @@
 
 #define MAX_CPUS	64
 
-#if defined(AIXv3r1) || defined(AIXv3r2)
+#if AIX_NUM < 4000000
 struct cpuinfo {
   int dummy;
 };
