@@ -3,7 +3,7 @@
 **     getkmemdata.c
 **
 ** Copyright (c) 1991-1995 Jussi Maki, All Rights Reserved.
-** Copyright (c) 1993-1998 Marcel Mol, All Rights Reserved.
+** Copyright (c) 1993-2001 Marcel Mol, All Rights Reserved.
 ** NON-COMMERCIAL USE ALLOWED. YOU ARE FREE TO DISTRIBUTE
 ** THIS PROGRAM AND MODIFY IT AS LONG AS YOU KEEP ORIGINAL
 ** COPYRIGHTS.
@@ -12,6 +12,7 @@
 #include <unistd.h>
 #include <stdio.h>
 #include <fcntl.h>
+#include "config.h"
 #include "getkmemdata.h"
 
 int kmemfd = -1;
@@ -37,6 +38,15 @@ int getkmemdata(void *buf, int bufsize, caddr_t address)
          */
         setuid(getuid());
         setgid(getgid());
+
+	/*
+	 * Set the close on exec bit for the kmem file descriptor.
+	 */
+        if (fcntl(kmemfd, F_SETFD, FD_CLOEXEC) == -1) {
+            perror("fcntl kmem");
+            close(kmemfd);
+            exit(1);
+        }
     }
     /*
      * Get the structure from the running kernel.
