@@ -3,7 +3,7 @@
 **     monitor.c
 **
 ** Copyright (c) 1991-1995 Jussi Maki, All Rights Reserved.
-** Copyright (c) 1993-1998 Marcel Mol, All Rights Reserved.
+** Copyright (c) 1993-2001 Marcel Mol, All Rights Reserved.
 ** NON-COMMERCIAL USE ALLOWED. YOU ARE FREE TO DISTRIBUTE
 ** THIS PROGRAM AND MODIFY IT AS LONG AS YOU KEEP ORIGINAL
 ** COPYRIGHTS.
@@ -21,15 +21,17 @@
 **         Marcel J.E. Mol
 **         Netherlands Europe
 **         E-mail: marcel@mesa.nl
-**         Phone (home): (+31) 015-3101310
+**         Phone (home): +31-6-54724868
 **
 **
 */
 
-#include "monitor.h"
-#include "print.h"
 #include <string.h>
 #include <strings.h>
+#include <sys/var.h>
+#include "config.h"
+#include "monitor.h"
+#include "print.h"
 
 
 #define MAXTOPCPU NPROCS
@@ -52,7 +54,7 @@ int synctime     = 0;
 
 char * progname;
 char   hostnm[128];
-struct utsname unames;
+int aix_version, aix_release, aix_level, aix_fix;
 
 /*
  * Main indications for interactive monitoring
@@ -68,6 +70,7 @@ int show_top_running = 0;    /* show process top list */
 int show_disk_full   = 0;    /* show all disk info */
 int show_inet_full   = 0;    /* show more network info */
 int show_nfs         = 0;    /* show nfs traffic info */
+int have_nfs         = 1;    /* assume NFS is configured on this system */
 #if defined(HAVE_AFS)
 int show_afs         = 0;    /* show afs traffic info */
 #endif
@@ -81,6 +84,7 @@ int valuecol         = COLOR_PAIR(7); /* metric value color */
 int namecol          = 0; /* metric name color */
 int valuecol         = 0; /* metric value color */
 #endif
+int check_badmem     = 0;    /* take vmk badmem into account */
 int sort_disks       = 0;    /* sort disk prints on read+write rate */
 int include_root     = 0;    /* include root sessions in user calculations */
 int logmode	     = 0;    /* log interval based info to file */
@@ -90,11 +94,12 @@ int dump_sample      = 0;    /* also log sample data to file */
 int do_curses        = 0;    /* will curses be used */
 int do_color         = 0;    /* will curses be used */
 int compress         = 0;    /* compress old log files */
+int normcpu          = 0;    /* normalize cpu% in top (smp relevant only) */
 int g_count          = 0;    /* if positive, only do g_count intervals */
 int current          = 0;    /* indicates current set of variables */
 int disk_cnt;                /* number of disk measured */
 int net_cnt;                 /* number of network interfaces measured */
-int cpu_cnt;                  /* number of cpus measured */
+int cpu_cnt;                 /* number of cpus measured */
 
 #ifdef COMPPROG
 char *compprog = COMPPROG;
@@ -120,6 +125,7 @@ int topflag_showusername = 1;
 int topflag_sort = TOPSORT_CPU;
 topcpu_t  topcpu[MAXTOPCPU];
 int ntop = MAXTOPCPU; /* top procs to show by default */
+int ndisk = 0; /* top procs to show by default */
 int nprocs; /* number of procs found */
 
 double time1, time2, timea[3];    /* for sample, previous sample and */
@@ -127,13 +133,18 @@ struct sysinfo si[3];             /*   previous interval */
 struct cpuinfo *cpus[3];
 struct vminfo vm[3];
 struct vmker vmk;
-struct dkstat *dk[3][MAX_DISKS];
+struct dkstat **dk[3];
 struct ifnet *ifnets[3][MAX_IFNETS];
 t_nfs_stats nfs[3];
 #if defined(HAVE_AFS)
 afsstat_t afs[3];
 #endif
 
+#if AIX_NUM > 4030000
+struct vario sys_vars;
+int disk_busy_flag;
+#endif
+
 extern time_t active_threshold;
 
 
@@ -161,7 +172,7 @@ char *termdef(int FileDescriptor, char Characteristic);
 
 /**************************************************************************/
 
-void
+int
 main(int argc, char **argv)
 {
     time_t nextinterval = 0, nextsample = 0;
@@ -172,7 +183,16 @@ main(int argc, char **argv)
     progname = *argv++;        /* go point to arguments */
 
     gethostname(hostnm, sizeof(hostnm));
-    uname(&unames);
+    if (bos_level(&aix_version, &aix_release, &aix_level, &aix_fix)) {
+        struct utsname unames;
+        /* fprintf(stderr,"%s: cannot determina AIX level using odm\n", progname); */
+        uname(&unames);
+        aix_version = atoi(unames.version);
+        aix_release = atoi(unames.release);
+        aix_level   = 0;
+        aix_fix     = 0;
+    }
+
     setlocale(LC_NUMERIC, "C"); /* be sure that decimal-point is "." */
 
 #if defined(DEBUG)
@@ -192,6 +212,9 @@ main(int argc, char **argv)
                 case 'A': /* -A[active_threshold]  for active user calculation */
                       active_threshold = atoi(*++argv);
 	              break;
+	        case 'b': /* -b[admem] */
+	              check_badmem = 1;
+	              break;
 #if defined(HAVE_AFS)
 	        case 'B': /* AFS */
 	              show_afs = 1;
@@ -209,8 +232,10 @@ main(int argc, char **argv)
 	        case 'd': /* -d[isk] */
 	              show_disk_full = 1;
 	              break;
-	        case 'D': /* -D[isk] */
+	        case 'D': /* -D[isk] [# disks to show] */
                       sort_disks = 1;
+	              if (*(argv+1) && **(argv+1) != '-')
+	                  ndisk = atoi(*++argv);
 	              break;
 	        case 'F': /* -F[ilesys] */
                       logallfs = 1;
@@ -247,7 +272,8 @@ main(int argc, char **argv)
                 case 'l': /* -l[ogmode] logfilename */
 	              logmode = 1;
 	              synctime = 1; /* force sync mode */
-                      if (**(++argv) == '-') {
+                      ++argv; /* nee to look at next arg  for filename */
+                      if (!*argv || **argv == '-') {
                           fprintf(stderr, "Logmode needs file name\n");
                           usage();
                       }
@@ -266,13 +292,13 @@ main(int argc, char **argv)
 	        case 'n': /* -n[et]  */
 	              show_inet_full = 1;
 	              break;
-	        case 'N': /* -n[fs] */
+	        case 'N': /* -N[fs] */
 	              show_nfs = 1;
 #if defined(HAVE_AFS)
 	              show_afs = 0;
 #endif
 	              break;
-		case 'r':
+		case 'r': /* r[unning] */
                       show_top_running = 1;
 	              break;
 		case 'R': /* -R[ootinclude] */
@@ -304,6 +330,9 @@ main(int argc, char **argv)
 	        case 'u': /* -u   ... dont show user names in top */
 	              topflag_showusername = 0;
 	              break;
+                case 'z': /* normalize cpu%  for smp systems*/
+                      normcpu = 1;
+                      break;
 	        default:
                       fprintf(stderr, "Something wrong with %s\n", *argv);
                       usage();
@@ -316,7 +345,7 @@ main(int argc, char **argv)
     /*
      * we want to do something ...
      */
-    if (!logmode && !show_sys_summary) {
+    if (!(logmode || show_sys_summary)) {
         show_sys = 1;
 #if defined(HAVE_AFS)
         if (!show_afs)
@@ -337,7 +366,7 @@ main(int argc, char **argv)
         sleep_sec = DEFAULT_SAMPLE;
 
     /*
-     * sample time may not larger that interval time...
+     * sample time may not larger than interval time...
      */
     if (logmode && sleep_sec > interval_sec)
         sleep_sec = interval_sec;
@@ -364,7 +393,7 @@ main(int argc, char **argv)
 #endif
     }
     else
-        nextsample = time(0) + 1; /* to get something to display soon */
+        nextsample = time(0) + sleep_sec/3 + 1; /* to get something to display soon */
 
     /*
      * init network and disk arrays
@@ -372,24 +401,60 @@ main(int argc, char **argv)
     for (i = 0; i < MAX_IFNETS; i++)
         ifnets[0][i] = ifnets[1][i] = ifnets[2][i] = 0;
 
-    for (i = 0; i < MAX_DISKS; i++)
-        dk[0][i] = dk[1][i] = dk[2][i] = 0;
 
+    /*
+     * Last chance to do things with root privs!
+     */
+    init_sys_vm();  /* if sysconfig is used we need root! */
+    init_nfs();     /* if sysconfig is used we need root! */
 
+#if AIX_NUM > 4030000
+    /*
+     * Since AIX 4.3 (or maybe some time erlier/later...) counters for
+     * disk busy are not on by default. Using sys_parm we can do tht ourselfes
+     * Apperantly there is a performance hit??? so check old stat and restore
+     * that when monitor exits.
+     */
+
+    sys_parm(SYSP_GET, SYSP_V_IOSTRUN, &sys_vars);
+    disk_busy_flag = sys_vars.v.v_iostrun.value;
+    if (!disk_busy_flag) {
+        sys_vars.v.v_iostrun.value = 1;
+        sys_parm(SYSP_SET, SYSP_V_IOSTRUN, &sys_vars);
+    }
+#endif
+   
     /*
      * get first time data
      * /dev/kmem will be opened as root. After that the euid is reset to the
      * user starting the program. So it is quite save to open files and
      * run programs from now on!
      */
+    
+    init_dkstat();
+    for (i = 0; i < 3; i++) {
+        if ((dk[i] = (struct dkstat **) calloc(dk_cnt, sizeof(struct dkstat *))) == NULL) {
+            fprintf(stderr, "init dk array failed for entry %d\n", i);
+            perror("calloc");
+        }
+    }
+
     get_sys_vm_info(&si[current], &vmk, &vm[current]);
+#if 0
     if (show_smp || logmode)
+#endif
         cpu_cnt   = get_cpuinfo(&cpus[current]);
+#if 1
+	/* this hopefully will remove the initial screen mess */
+        cpu_cnt   = get_cpuinfo(&cpus[current^1]);
+#else
+        memcpy(cpus[current], cpus[current^1], sizeof(struct cpuinfo) * cpu_cnt);
+#endif
     if (show_sys || show_disk_full || logmode) 
-        disk_cnt = get_dkstat(dk[current]);
+        /* disk_cnt = */ get_dkstat(dk[current]);
     if (show_sys || show_inet_full || logmode)
         net_cnt  = get_ifnet(ifnets[current]);
-    if ((show_sys && show_nfs) || logmode)
+    if (have_nfs && ((show_sys && show_nfs) || logmode))
         get_nfsstat(&nfs[current]);
 #if defined(HAVE_AFS)
     if ((show_sys && show_afs) || logmode)
@@ -401,6 +466,7 @@ main(int argc, char **argv)
     timea[current] = realtime();
 
     if (logmode) {
+        init_log_dkstat(); 
         /*
          * setup initial logmode variables, and setup umask for logfiles
          */
@@ -456,8 +522,8 @@ main(int argc, char **argv)
 
 #if defined(DEBUG)
         OPENDB;
-        fprintf(debugfp, "In synctime loop: now=%ld.%ld, nextsample=%ld\n",
-                             tp.tv_sec, tp.tv_usec, nextsample);
+        fprintf(debugfp, "In synctime loop: now=%ld.%ld, nextsample=%ld, current=%d\n",
+                             tp.tv_sec, tp.tv_usec, nextsample, current);
         CLOSEDB;
 #endif
         wait_input_or_sleep(nextsample - tp.tv_sec - 1, 1000000 - tp.tv_usec);
@@ -532,7 +598,8 @@ main(int argc, char **argv)
                 if (show_smp)
                     print_cpuinfo(refresh_time, cpus[current], cpus[current^1]);
 	        if (show_disk_full)
-	            print_dkstat_full(refresh_time, dk[current], dk[current^1]);
+	            print_dkstat_full(refresh_time, dk[current], dk[current^1],
+                                      ndisk);
 	        if (show_inet_full)
 	            print_ifnet(refresh_time, ifnets[current],
                                               ifnets[current^1]);
@@ -549,7 +616,7 @@ main(int argc, char **argv)
 #endif
                 getyx(stdscr, t_row, t_col);
                 c_row = max(c_row, t_row);
-	        print_dkstat(refresh_time, dk[current], dk[current^1]);
+	        print_dkstat(refresh_time, dk[current], dk[current^1], ndisk);
                 getyx(stdscr, t_row, t_col);
                 c_row = max(c_row, t_row);
 	        print_ifnet(refresh_time, ifnets[current], ifnets[current^1]);
@@ -557,13 +624,16 @@ main(int argc, char **argv)
             if ((show_sys && show_top) || (show_sys_summary && show_top_a)) {
                 getyx(stdscr, t_row, t_col);
                 c_row = max(c_row, t_row);
-	        print_topcpu(refresh_time, topcpu, min(ntop, nprocs),
-                             timea[current], c_row+1);
+                if (c_row < numlines - 2)
+	            print_topcpu(refresh_time, topcpu, min(ntop, nprocs),
+                             timea[current], c_row+1, cpu_cnt);
             }
             getyx(stdscr, t_row, t_col);
             c_row = max(c_row, t_row);
-	    move(c_row+1, 0);
-            clrtobot();
+            if (c_row < numlines - 1) {
+	        move(c_row+1, 0);
+                clrtobot();
+            }
 	    move(0, 0);
 	    refresh();
         }
@@ -605,7 +675,7 @@ main(int argc, char **argv)
 
 
             /*
-             * if log the interval ended, dump the data
+             * if log interval ends, dump the data
              */
 	    if (timea[current] >= nextinterval - 1) {
 #if defined(DEBUG)
@@ -626,8 +696,8 @@ main(int argc, char **argv)
                                                    &tmp_tm);
                     if (g_count > 0)
                         sprintf(ts+strlen(ts), " s%d", g_count);
-                    fprintf(logfp, "interval %s %d %s\n", hostnm,
-                                   interval_sec, ts);
+                    fprintf(logfp, "interval %s %d %s %d\n", hostnm,
+                                   interval_sec, ts, sleep_sec);
                     print_log(timea[current] - timea[2],
                               &si[current], &si[2], &vmk,
                               &vm[current], &vm[2],
@@ -672,7 +742,8 @@ main(int argc, char **argv)
      
     }
 
-    /* NOTREACHED */
+    /* NOT REACHED */
+
     if (do_curses)
        endwin();
 
@@ -691,7 +762,7 @@ usage()
 {
 
     fprintf(stderr,"Usage: %s [options]\n\n", progname);
-    fprintf(stderr,"%s\n(c) 1991-1998 Jussi Maki (jmaki@csc.fi), "
+    fprintf(stderr,"%s\n(c) 1991-2001 Jussi Maki (jmaki@csc.fi), "
                    "Marcel Mol (marcel@mesa.nl)\n\n", MONITOR_NAME);
     fprintf(stderr,"Monitor is a curses-based AIX system event monitor\n");
     fprintf(stderr,"The following options apply:\n");
@@ -703,8 +774,9 @@ usage()
    "   -Sync         synchronise sample and interval time on day boundary\n"
    "   -smp          show SMP (multiprocessor) info in alternative display\n"
    "   -disk         show detailed disks info in alternative display\n"
-   "   -D            sort disk info on read+write rate\n"
+   "   -D [cnt]      sort disk info on read+write rate, show cnt disks\n"
    "   -net          show detailed network info in alternative display\n"
+   "   -badmem       take bad memory into account for total memory count\n"
    "   -Nfs          show NFS information"
 #if defined(HAVE_AFS)
    " (instead of AFS)\n"
@@ -717,6 +789,7 @@ usage()
    "   -log logfile  log mode, dump interval date to logfile. Logfile is used\n"
    "                 as a strftime format string, based on start of interval.\n"
    "                 The logfile will be opened/closed for every dump.\n"
+   "   -z            normalize cpu% for top processes to 100%\n"
    "   -L            also dump samples to logfile\n"
    "   -Toplog       also log top records to logfile\n"
    "   -F            also log non-local filesystem information (e.g. NFS)\n"
@@ -727,7 +800,7 @@ usage()
    "   -Highlight [none|bold|reverse%s] highlighting headers (default %s)\n"
 
    "\nPress h or ? in interactive mode to see its interactive commands\n"
-   "This version of monitor is compiled for AIX %d.%d\n",
+   "This version of monitor is compiled on AIX %d.%d.%d.%d\n",
 
    DEFAULT_SAMPLE, DEFAULT_INTERVAL_SAMPLE, DEFAULT_INTERVAL,
    compprog,
@@ -739,7 +812,7 @@ usage()
    highlight == A_STANDOUT ? "reverse"
               : highlight == A_BOLD ? "bold"
               : highlight ? "color" : "off",
-    AIX_VERSION, AIX_RELEASE);
+    AIX_VERSION, AIX_RELEASE, AIX_LEVEL, AIX_FIX);
 
     exit(0);
 
@@ -760,7 +833,7 @@ copy_log_variables()
     afs[2]   = afs[current];
 #endif
     if (!cpus[2])
-        cpus[2] = (struct cpuinfo *)calloc(sizeof(struct cpuinfo), cpu_cnt);
+        cpus[2] = (struct cpuinfo *)calloc(cpu_cnt, sizeof(struct cpuinfo));
     memcpy(cpus[2], cpus[current], sizeof(struct cpuinfo) * cpu_cnt);
     for (i = 0; i < dk_cnt; i++) {
         if (!dk[2][i])
@@ -801,42 +874,55 @@ openlogfile(struct tm *tim)
         fprintf(debugfp, "\tgonna print headers\n");
         CLOSEDB;
 #endif
-        stat(logfile, &st_buf);
-        if (st_buf.st_size == 0)
-            print_log_header();
         if (compress && *logfile_old != '\0') {
+            int child_stat;
             /*
              * Compress file in a seperate process!
              */
             if (!fork()) {
-                char buf[256];
-                /*
-                 * child will compress the old logfile
-                 */
-                if (do_curses) { /* not daemon mode */
-                    fclose(stdin);
-                    fclose(stdout);
-                    fclose(stderr);
-                }
+                fclose(logfp);
                 setpgrp(); /* detach from parent */
-                strcpy(buf, compprog);
-                strcat(buf, " ");
-                strcat(buf, logfile_old);
                 /*
-                 * Should check if we are sh-ish or csh-ish ???
+                 * Create child that does the actual compression
+                 * so our parent can wait() for us.
+                 * This will avoid defunct processes.
+                 * There must be a better way to do this...
                  */
+                if (!fork()) {
+                    char buf[256];
+                    /*
+                     * child will compress the old logfile
+                     */
+                    if (do_curses) { /* not daemon mode */
+                        fclose(stdin);
+                        fclose(stdout);
+                        fclose(stderr);
+                    }
+                    setpgrp(); /* detach from parent */
+                    strcpy(buf, compprog);
+                    strcat(buf, " ");
+                    strcat(buf, logfile_old);
+                    /*
+                     * Should check if we are sh-ish or csh-ish ???
+                     */
 #if defined(DEBUG)
-                strcat(buf, "> /tmp/moncompress.log 2>&1");
+                    strcat(buf, "> /tmp/moncompress.log 2>&1");
 #else
-                strcat(buf, "> /dev/null 2>&1");
+                    strcat(buf, "> /dev/null 2>&1");
 #endif
-                exit(system(buf));
+                    exit(system(buf));
+                }
+                exit(0);
             }
+            wait(&child_stat);
         }
         /*
          * Now remember the new filename
          */
         strcpy(logfile_old, logfile);
+        fstat(fileno(logfp), &st_buf);
+        if (st_buf.st_size == 0)
+            print_log_header();
     }
 
 #if defined(DEBUG)
@@ -936,6 +1022,14 @@ start_screen()
         numcolumns = 80;
     if (numlines == 0)
         numlines = 24;
+    if (numlines < 24 || numcolumns < 79) {
+        fprintf(stderr, "Monitor needs at least 24x79 screen. You currently use %dx%d\n",
+                         numlines, numcolumns);
+       exit(1);
+    }
+    /*
+     * If the screen is wide enough we show some more info in top
+     */
     topflag_usersystemtime = numcolumns > 94 ? 1 : 0; 
     initscr();
 #if defined(_HAS_COLOR)
@@ -973,6 +1067,13 @@ quit_monitor()
         endwin();
     }
 
+#if AIX_NUM > 4030000
+    if (!disk_busy_flag) {
+        sys_vars.v.v_iostrun.value = 0;
+        sys_parm(SYSP_SET, SYSP_V_IOSTRUN, &sys_vars);
+    }
+#endif
+
     exit(0);
 
 } /* quit_monitor */
@@ -1003,6 +1104,9 @@ checkinput()
             case 'a' : show_sys ^= 1;
                        show_sys_summary = !show_sys;
                        initscreen();
+                       break;
+            case 'b' : check_badmem ^= 1;
+                       break;
 #if defined(HAVE_AFS)
             case 'B' : show_afs ^= 1;
                        if (show_afs) {
@@ -1017,7 +1121,7 @@ checkinput()
                        break;
             case 'd' : show_disk_full ^= 1;
                        if (show_disk_full && show_sys_summary)
-                           disk_cnt  = get_dkstat(dk[current]);
+                           /* disk_cnt  = */ get_dkstat(dk[current]);
                        if (show_sys_summary)
                            initscreen();
                        break;
@@ -1025,6 +1129,8 @@ checkinput()
             case '?' : display_help();
                        read(fileno(stdin), inbuf, 1);
                        initscreen();
+                       /* init again to resolve inverse screen problem... */
+                       initscreen();
                        break;
             case 'm' : topflag_sort = TOPSORT_RES;
                        break;
@@ -1034,21 +1140,31 @@ checkinput()
                        if (show_sys_summary)
                            initscreen();
                        break;
-            case 'N' : show_nfs ^= 1;
-                       if (show_nfs) {
-                           get_nfsstat(&nfs[current]);
+            case 'N' : if (have_nfs) {
+                           show_nfs ^= 1;
+                           if (show_nfs) {
+                               get_nfsstat(&nfs[current]);
 #if defined(HAVE_AFS)
-                           show_afs = 0;
+                               show_afs = 0;
 #endif
+                           }
+                           if (show_sys)
+                               initscreen();
                        }
-                       if (show_sys)
-                           initscreen();
                        break;
             case 'p' : topflag_sort = TOPSORT_PAGE;
                        break;
             case 's' : show_smp ^= 1;
+#if 0
                        if (show_smp) /* need to get get cpu_cnt now! */
                            cpu_cnt = get_cpuinfo(&cpus[current]);
+                           cpu_cnt = get_cpuinfo(&cpus[current^1]);
+#endif
+#if defined(DEBUG)
+        		OPENDB;
+        		fprintf(debugfp, "setting smp current=%d\n", current);
+        		CLOSEDB;
+#endif
                        if (show_sys_summary)
                            initscreen();
                        break;
@@ -1068,6 +1184,8 @@ checkinput()
                        break;
             case 'v' : topflag_sort = TOPSORT_VIR;
                        break;
+            case 'z' : normcpu ^= 1;
+                       break;
             case '-' : sleep_sec--;
                        if (sleep_sec < 1)
                            sleep_sec = 1;
@@ -1085,6 +1203,11 @@ checkinput()
                            sleep_sec = interval_sec;
                        break;
         }
+#if defined(DEBUG)
+	OPENDB;
+	fprintf(debugfp, "processed %d / %c command\n", inbuf[0], inbuf[0]);
+	CLOSEDB;
+#endif
     }
 
     return;
@@ -1098,34 +1221,37 @@ display_help()
 {
   clear();
   BOLDON;
-  printw("monitor -- %s\ncompiled on AIX %d.%d running on AIX %s.%s\n\n",
-          MONITOR_NAME, AIX_VERSION, AIX_RELEASE,
-          unames.version, unames.release);
-  BOLDOFF;
+  printw("monitor -- %s\ncompiled on AIX %d.%d.%d.%d running on AIX %d.%d.%d.%d\n",
+          MONITOR_NAME, AIX_VERSION, AIX_RELEASE, AIX_LEVEL, AIX_FIX,
+          aix_version, aix_release, aix_level, aix_fix);
+  move(2,8); printw("RESTRICTED FOR NON-COMMERCIAL USE ONLY\n");
+  BOLDOFF;                                                                                        
   NAMECOLON;
-  printw("(C) Copyright 1991-1998 Marcel Mol (marcel@mesa.nl), Jussi Maki (jmaki@csc.fi)\n\n");
+  printw("(C) Copyright 1991-2001 Marcel Mol (marcel@mesa.nl), Jussi Maki (jmaki@csc.fi)\n");
+  printw("MESA Consulting, Unix and Internet Technology Specialist, http://www.mesa.nl\n\n");
   printw("Command toggles available in full screen mode:\n");
   printw("\n");
-  printw("    ^L             refresh screen\n");
-  printw("     a             display alternative screen\n");
-  printw("     d             display detailed disk info\n");
-  printw("     n             display detailed network info\n");
-  printw("     N             display NFS info (only on global screen)\n");
+  printw(" ^L         refresh screen           q       quit monitor\n");
+  printw("  a         switch between screens   h ?     this help screen\n");
+  printw("  d         detailed disk info       n       detailed network info\n");
+  printw("  s         SMP cpuinfo              N       NFS info\n");
+  printw("  b         include bad memory       z       normalize top CPU%\n");
 #if defined(HAVE_AFS)
-  printw("     B             display AFS info (only on global screen)\n");
+  printw("  B         AFS info\n");
 #endif
-  printw("     s             display SMP multiprocessor cpuinfo\n");
-  printw("     t             display top cpu processes\n");
-  printw("     u             display username/userid in top\n");
-  printw("     r             top only displays running processes\n");
-  printw("     c, p, m, v    sort top on cpu, pagefautls, resident mem, virtual mem\n");
-  printw("     R             include root in user count/activity\n");
-  printw("     +, -          increase, decrease sample time by 1 second\n");
-  printw("     >, <          increase, decrease sample time by 5 seconds\n");
-  printw("     q             quit monitor\n");
-  printw("     h or ?        this message\n");
+  printw("  t         top processes            u      username/userid in top\n");
+  printw("  r         top only displays running processes\n");
+  printw("  c p m v   sort top on cpu, pagefaults, resident mem, virtual mem\n");
+  printw("  R         include root in user count/activity\n");
+  printw("  + - (> <) increase, decrease sample time by 1 (5) second(s)\n");
   printw("\n");
-  printw("The a, d, n and s command can be displayed together\n");
+  printw("The d, n and s keys only show up in the alternative screen\n");
+#if defined(HAVE_AFS)
+  printw("The N and B keys only show");
+#else
+  printw("The N key only shows");
+#endif
+  printw(" up in the global screen\n");
   printw("\n");
   NAMECOLOFF;
   BOLDON;
