@@ -3,7 +3,7 @@
 **     monitor.h
 **
 ** Copyright (c) 1991-1995 Jussi Maki, All Rights Reserved.
-** Copyright (c) 1993-1998 Marcel Mol, All Rights Reserved.
+** Copyright (c) 1993-2001 Marcel Mol, All Rights Reserved.
 ** NON-COMMERCIAL USE ALLOWED. YOU ARE FREE TO DISTRIBUTE
 ** THIS PROGRAM AND MODIFY IT AS LONG AS YOU KEEP ORIGINAL
 ** COPYRIGHTS.
@@ -36,6 +36,7 @@
 #include "print_nfsstat.h"
 #include "print_afsstat.h"
 #include "getloadavg.h"
+#include "get_odm.h"
 #include "get_top.h"
 #include "print_top.h"
 #include "get_sysvminfo.h"
@@ -43,8 +44,11 @@
 #include "filesys.h"
 #include "monprint.h"
 
+#if defined(DEBUG)
+#include "debug.h"
+#endif
 
-#define MONITOR_NAME "AIX System monitor v" MON_VERSION " " MON_DATE
+#define MONITOR_NAME "AIX Monitor v" MON_VERSION " " MON_DATE
 
 #if !defined(AIX_VERSION)
 # define AIX_VERSION 0
@@ -52,14 +56,18 @@
 #if !defined(AIX_RELEASE)
 # define AIX_RELEASE 0
 #endif
+#if !defined(AIX_LEVEL)
+# define AIX_LEVEL 0
+#endif
+#if !defined(AIX_FIX)
+# define AIX_FIX 0
+#endif
+#if !defined(AIX_NUM)
+# define AIX_NUM  AIX_VERSION 0 AIX_RELEASE 0 AIX_LEVEL 0 AIX_FIX
+#endif
 
 
 #if !defined(HIGHLIGHT)
-#define HIGHLIGHT 1
+# define HIGHLIGHT 1
 #endif
 
-#if defined(DEBUG)
-extern FILE *debugfp;
-#define OPENDB  debugfp = fopen("monitor.log", "a")
-#define CLOSEDB fclose(debugfp)
-#endif
