@@ -3,18 +3,19 @@
 **     monprint.c
 **
 ** Copyright (c) 1991-1995 Jussi Maki, All Rights Reserved.
-** Copyright (c) 1993-1998 Marcel Mol, All Rights Reserved.
+** Copyright (c) 1993-2001 Marcel Mol, All Rights Reserved.
 ** NON-COMMERCIAL USE ALLOWED. YOU ARE FREE TO DISTRIBUTE
 ** THIS PROGRAM AND MODIFY IT AS LONG AS YOU KEEP ORIGINAL
 ** COPYRIGHTS.
 */
 
-#include "monitor.h"
 #include <sys/times.h>
 #include <string.h>
 #include <sys/stat.h>
 #include <sys/param.h>
 #include <utmp.h>
+#include "config.h"
+#include "monitor.h"
 #include "print.h"
 
 typedef struct {
@@ -46,13 +47,18 @@ extern int logtop;
 extern int numcolumns;
 
 extern FILE *logfp;
-extern struct utsname unames;
 
+extern int aix_version;
+extern int aix_release;
+extern int aix_level;
+extern int aix_fix;
 
 time_t uptime;
 time_t active_threshold = 60;
 
-#define UTMP_FILENAME    "/etc/utmp"
+#ifndef UTMP_FILE
+# define UTMP_FILE        "/etc/utmp"
+#endif
 #define	MINUTES          60
 #define HOURS            (MINUTES * 60)
 #define DAYS             (HOURS * 24)
@@ -310,8 +316,8 @@ void get_users(nusers_t *users)
     time_t now, active_threshold = 60;
     time_t last_access, last_change, last_mod, inact, rootinact;
   
-    if (!(utmpf = fopen(UTMP_FILENAME, "r"))) {
-        perror(UTMP_FILENAME);
+    if (!(utmpf = fopen(UTMP_FILE, "r"))) {
+        perror(UTMP_FILE);
         users->nusers = -1;
         return;
     }
@@ -537,29 +543,37 @@ print_sysinfo(double refresh_time, time_t now,
     move(y + 4, x); printw("%6.1f", pgspgins);
     move(y + 5, x); printw("%6.1f", pgspgouts);
 
+#define myprw1(a) if (a < 100000) printw("%5d", (int) a ); \
+                  else printw("%4dk", (int) a / 1000);
+
     x = 48; y = 8;
-    move(y + 1, x); printw("%5d", (int) pswitch);
-    move(y + 2, x); printw("%5d", (int) syscall);
-    move(y + 3, x); printw("%5d", (int) sysread);
-    move(y + 4, x); printw("%5d", (int) syswrite);
-    move(y + 5, x); printw("%5d", (int) sysfork);
-    move(y + 6, x); printw("%5d", (int) sysexec);
-    move(y + 7, x); printw("%5d", (int) rcvint);
-    move(y + 8, x); printw("%5d", (int) xmtint);
+    move(y + 1, x); myprw1(pswitch);
+    move(y + 2, x); myprw1(syscall);
+    move(y + 3, x); myprw1(sysread);
+    move(y + 4, x); myprw1(syswrite);
+    move(y + 5, x); myprw1(sysfork);
+    move(y + 6, x); myprw1(sysexec);
+    move(y + 7, x); myprw1(rcvint);
+    move(y + 8, x); myprw1(xmtint);
 #ifdef mon_notdef
-    move(y + 9, x); printw("%5d", (int) mdmint);
+    move(y + 9, x); myprw1(mdmint);
 #endif
     
+#define MB  1048576
+#define KB  1024
+#define myprw2(a) if (a < 100000) printw("%7d", (int) a ); \
+                  else if (a < 100000*KB) printw("%6dk", (int) a / KB); \
+                  else printw("%6dM", (int) a / MB);
+
     x = 62; y = 8;
     move(y + 1, x); printw("%7d", (int) si_iget);
     move(y + 2, x); printw("%7d", (int) si_namei);
     move(y + 3, x); printw("%7d", (int) si_dirblk);
-    move(y + 4, x); printw("%7d", (int) si_readch);
-    move(y + 5, x); printw("%7d", (int) si_writech);
+    move(y + 4, x); myprw2(si_readch);
+    move(y + 5, x); myprw2(si_writech);
     move(y + 6, x); printw("%7d", (int) si_rawch);
     move(y + 7, x); printw("%7d", (int) si_canch);
     move(y + 8, x); printw("%7d", (int) si_outch);
-
     return;
 
 } /* print_sysinfo */
@@ -571,8 +585,10 @@ print_summary_init()
 {
 
     BOLDON;
-    move(0, 45); printw("%s", hostnm);
+    move(0, 35); printw("%s", hostnm);
     move(0,  0); printw("Load averages:");
+    move(1,  60); printw("For non-commercial");
+    move(2,  66); printw("use only!");
     move(1,  0); printw("Cpu states:");
     move(2,  0); printw("Logged on:");
     move(3,  0); printw("Real memory:");
@@ -607,7 +623,7 @@ print_summary(double refresh_time, time_t now,
      * cpu-states
      */
     move(1, 12); printw("%4.1f%% user %4.1f%% system "
-                        "%4.1f%% wait %4.1f%% idle   ",
+                        "%4.1f%% wait %4.1f%% idle  ",
 	                cpu_user, cpu_sys, cpu_wait, cpu_idle);
     /*
      * logged on users
@@ -647,13 +663,14 @@ print_log_header()
 {
 
     fprintf(logfp, "# Monitor -- %s\n", MONITOR_NAME);
-    fprintf(logfp, "#            compiled on AIX %d.%d running on AIX %s.%s\n",
-          AIX_VERSION, AIX_RELEASE, unames.version, unames.release);
+    fprintf(logfp, "#            compiled on AIX %d.%d.%d.%d running on AIX %d.%d.%d.%d\n",
+          AIX_VERSION, AIX_RELEASE, AIX_LEVEL, AIX_FIX,
+          aix_version, aix_release, aix_level, aix_fix);
     fprintf(logfp, "#interval/sample hostname interval_len startime "
                    "endtime dayname day month year monthname weekdaynum"
                    " weeknum, sunday is day 0\n");
     fprintf(logfp, "# cpubusy[_max] sys%% wait%% user%% idle%%\n");
-    fprintf(logfp, "# runque[_max] runnable/sample runnable/sec\n");
+    fprintf(logfp, "# runque[_max] runnable/sample runnable/sec runcount runocccount\n");
     fprintf(logfp, "# users[_max] number_of_users active_users network_remote"
                    " total_inactive_secs number_of_rootusers active_rootusers"
                    " network_rootremote total_rootinactive_secs\n");
@@ -770,7 +787,8 @@ print_log(double refresh_time, struct sysinfo *si1, struct sysinfo *si2,
     /*
      * System run and swap load
      */
-    fprintf(logfp, "runque %.2f %.2f\n", runnable, runave);
+    fprintf(logfp, "runque %.2f %.2f %.2f %.2f\n",
+                   runnable, runave, runque_tmp, runocc_tmp);
 
     fprintf(logfp, "swapin %.2f %.2f\n", swapin, swapave);
 
@@ -925,6 +943,7 @@ print_log_max(struct dkstat *dk1[], struct ifnet *if1[])
 void
 initscreen()
 {
+    static have_mesa = 0;
 
     clear();
 
@@ -948,6 +967,16 @@ initscreen()
         if (show_afs)
             print_afsstat_init();
 #endif
+        if (!have_mesa) {
+            move(19,0); printw("MESA Consulting");
+            move(20,0); printw("Unix and Internet");
+            move(21,0); printw("Technology Specialists");
+            move(22,0); printw("http://www.mesa.nl");
+            have_mesa++;
+	}
+        BOLDON;
+        move(7,8); printw("FOR NON-COMMERCIAL USE ONLY     see http://www.mesa.nl/monitor");
+        BOLDOFF;
     }
 /*  nothing to do for top at this time ...
     if (show_top || show_top_a)
