@@ -3,7 +3,7 @@
 **     print_dkstat.h
 **
 ** Copyright (c) 1991-1995 Jussi Maki, All Rights Reserved.
-** Copyright (c) 1993-1998 Marcel Mol, All Rights Reserved.
+** Copyright (c) 1993-2001 Marcel Mol, All Rights Reserved.
 ** NON-COMMERCIAL USE ALLOWED. YOU ARE FREE TO DISTRIBUTE
 ** THIS PROGRAM AND MODIFY IT AS LONG AS YOU KEEP ORIGINAL
 ** COPYRIGHTS.
@@ -13,12 +13,15 @@ void calc_dkstat(double refresh_time, struct dkstat *dk1[],
                                       struct dkstat *dk2[]);
 void print_dkstat_init();
 void print_dkstat(double refresh_time, struct dkstat *dk1[],
-                                       struct dkstat *dk2[]);
+                                       struct dkstat *dk2[], int ndisk);
 void print_dkstat_full_init();
 void print_dkstat_full(double refresh_time, struct dkstat *dk1[],
-                                            struct dkstat *dk2[]);
+                                            struct dkstat *dk2[], int ndisk);
 void log_dkstat_header();
 void log_dkstat(double refresh_time, struct dkstat *dk1[],
                                      struct dkstat *dk2[]);
 void max_dkstat_init();
+void init_log_dkstat();
 void log_dkstat_max(struct dkstat *dk[]);
+
+extern int logmode;
