@@ -3,7 +3,7 @@
 **     print_top.h
 **
 ** Copyright (c) 1991-1995 Jussi Maki, All Rights Reserved.
-** Copyright (c) 1993-1998 Marcel Mol, All Rights Reserved.
+** Copyright (c) 1993-2001 Marcel Mol, All Rights Reserved.
 ** NON-COMMERCIAL USE ALLOWED. YOU ARE FREE TO DISTRIBUTE
 ** THIS PROGRAM AND MODIFY IT AS LONG AS YOU KEEP ORIGINAL
 ** COPYRIGHTS.
@@ -11,7 +11,7 @@
 
 void calc_top(double refresh_time, topcpu_t *top, int nproc);
 void print_topcpu(double refresh_time, topcpu_t *top, int ntop,
-                  time_t now, int y);
+                  time_t now, int y, int cpu_cnt);
 void log_top_header();
 void log_top(double refresh_time, topcpu_t *top, int nproc);
 void log_top_max();
